//+------------------------------------------------------------------+
//|                        COMMON DECLARATION                        |
//|     YOU SHOULD NOT HAVE TO MODIFY ANYTHING BELOW THIS BOX        |
//+------------------------------------------------------------------+
#include <MakeMagicNumber.mqh>

datetime timeprev=0;
bool EnterLong = false;
bool ExitLong  = false;
bool EnterShort = false;
bool ExitShort = false;
//+------------------------------------------------------------------+
//|                          EXPERT COLORS                           |
//|            ADJUST THESE VALUES TO MATCH REQUIREMENTS             |
//+------------------------------------------------------------------+
//---- This is the name that will appear on the chart if the expert 
//---- is attached
static string      ExpertName = "Trend Catcher";
//---- Enter Long Color
static color   EnterLongColor = Green;
//---- Enter Short Color
static color  EnterShortColor = Red;
//---- Exit Long Color
static color    ExitLongColor = Blue;
//---- Exit Short Color
static color   ExitShortColor = Pink;
//---- If using pending orders and if you need an expiration enter the value here
static int     EXPIRATION     = 0;

//+------------------------------------------------------------------+
//|                          EXPERT ID                               |
//+------------------------------------------------------------------+
// You need to give a unique id to each one of your expert
// This will be used as the magic number and then keep track of 
// the orders generated by the expert. 

//---- The expert ID has to be a 2 digit number followed by 0000
//---- For example: 550000 or 120000 etc ...
static int ExpertID = 0500000000;

//+------------------------------------------------------------------+
//|                           USER INPUT                             |
//|            ADJUST THESE VALUES TO MATCH REQUIREMENTS             |
//+------------------------------------------------------------------+
//|                Trading Inputs/Indicators Inputs                  |
//+------------------------------------------------------------------+
// If you do not need to use one of these inputs simply comment it out
// For example:
// extern bool Multiple entries = false;
 
extern double Lots = 0.1;
//---- Set stoploss to 0.0 if you don't want to use it
extern double StopLoss= 50.0;
//---- Set ProfitTarget to 0.0 if you don't want to use it
extern double ProfitTarget = 150.0;
//---- Set TrailingStop to 0.0 if you don't want to use it
extern double TrailingStop = 0.0;
//---- Slippage / Max deviation from quoted price
extern int    Slippage= 2;
//for a highest and lowest for the past x bars use 0, use a 1 if you just want it from the entry point
extern int StopMethod = 0;
//set this to how many bars you wish to look at for your highest and lowest value for your stop loss
extern int BarsForStopMethod0 = 5;

extern int Fast_EMA = 40;
extern int Slow_EMA = 50;
extern int Signal_SMA = 100;

//+------------------------------------------------------------------+
//|                    EXPERT BASIC INITIALISATION                   |
//|     YOU SHOULD NOT HAVE TO MODIFY ANYTHING BELOW THIS BOX        |
//+------------------------------------------------------------------+
int start()
  {
  
     if(timeprev==Time[0]){return(0);}
     timeprev = Time[0];
     
     int MagicNumber = 0;
     MagicNumber = MakeMagicNumber( ExpertID);
     
     EnterLong = false;
     ExitLong  = false;
     EnterShort = false;
     ExitShort = false;
     
     TrailingAlls(0,TrailingStop);
//+------------------------------------------------------------------+
//|                    EXPERT BASIC CALCULATION                      |
//|                          START HERE                              |
//+------------------------------------------------------------------+
double MACD0,OSMA0;
double MACD1,OSMA1;
MACD0 = iMACD(NULL,0,Fast_EMA,Slow_EMA,Signal_SMA,PRICE_CLOSE,MODE_MAIN,1);
MACD1 = iMACD(NULL,0,Fast_EMA,Slow_EMA,Signal_SMA,PRICE_CLOSE,MODE_MAIN,2);
OSMA0 = iOsMA(NULL,0,Fast_EMA,Slow_EMA,Signal_SMA,PRICE_CLOSE,1);
OSMA1 = iOsMA(NULL,0,Fast_EMA,Slow_EMA,Signal_SMA,PRICE_CLOSE,2);


if (MACD0<0 && MACD0>MACD1)
EnterLong=true;
if (MACD0>0 && MACD0<MACD1)
EnterShort=true;
if (OSMA0<0 && OSMA0>OSMA1)
EnterLong=true;
if (OSMA1>0 && OSMA0<OSMA1)
EnterShort=true;


//+------------------------------------------------------------------+
//|                    EXPERT BASIC CALCULATION                      |
//|                          END HERE                                |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|     YOU SHOULD NOT HAVE TO MODIFY ANYTHING BELOW THIS BOX        |
//+------------------------------------------------------------------+

// ENTER LONG CONDITION
if(EnterLong == true && CountLongs(MagicNumber)== 0)
{
   //CLOSE OPEN ORDER
   //CloseShorts(MagicNumber);
   // PLACE THE ORDER
   OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,StopLong(Bid,StopLoss),TakeLong(Ask,ProfitTarget),ExpertName,MagicNumber,EXPIRATION,EnterLongColor);
} 
// ENTER SHORT CONDITION
if(EnterShort == true && CountShorts(MagicNumber)== 0)
{
   //CLOSE OPEN ORDER
   //CloseLongs(MagicNumber);
   // PLACE THE ORDER
   OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,StopShort(Ask,StopLoss),TakeShort(Bid,ProfitTarget),ExpertName,MagicNumber,EXPIRATION,EnterShortColor);
} 
if(ExitLong == true && CountLongs(MagicNumber)== 1)
{
   //CLOSE OPEN ORDER
   CloseLongs(MagicNumber);
}
if(ExitShort == true && CountShorts(MagicNumber)== 1)
{
   //CLOSE OPEN ORDER
   CloseShorts(MagicNumber);
}
   return(0);
 }

//+------------------------------------------------------------------+
//| Calculate Stop Long                                              |
//+------------------------------------------------------------------+
double StopLong(double price,int stop)
{
double bottom = Low[iLowest(NULL,0,MODE_LOW,BarsForStopMethod0,1)];
 if(stop==0)
 {
  return(0);
 }
 if (StopMethod == 0)
 {
  return(bottom-(stop*Point));
 }
 else 
 {
 return (price-(stop*Point));
 }
}
//+------------------------------------------------------------------+
//| Calculate Stop Short                                             |
//+------------------------------------------------------------------+
double StopShort(double price,int stop)
{
double top = High[iHighest(NULL,0,MODE_HIGH,BarsForStopMethod0,1)];
 if(stop==0)
 {
  return(0);
 }
 if (StopMethod == 0)
 {
  return(top+(stop*Point));
 }
 else
 {
 return (price+(stop*Point));
 }
 
}
//+------------------------------------------------------------------+
//| Calculate Profit Target Long                                     |
//+------------------------------------------------------------------+
double TakeLong(double price,int take)
{
 if(take==0) {  return(0);}
 else        {  return(price+(take*Point));}
}
//+------------------------------------------------------------------+
//| Calculate Profit Target Short                                     |
//+------------------------------------------------------------------+
double TakeShort(double price,int take)
{
 if(take==0) {  return(0);}
 else        {  return(price-(take*Point));}
}
//+------------------------------------------------------------------+
//| Calculate concurrent Long position                               |
//+------------------------------------------------------------------+
int CountLongs(int MagicNumber)
{
 int count=0;
 int trade;
 int trades=OrdersTotal();
 for(trade=0;trade<trades;trade++)
 {
  OrderSelect(trade,SELECT_BY_POS,MODE_TRADES);
  
  if( OrderSymbol()!=Symbol() || OrderMagicNumber() != MagicNumber )
   continue;
   
  if(OrderType()==OP_BUY)
   count++;
 }//for
 return(count);
}
//+------------------------------------------------------------------+
//| Calculate concurrent short position                              |
//+------------------------------------------------------------------+
int CountShorts(int MagicNumber)
{
 int count=0;
 int trade;
 int trades=OrdersTotal();
 for(trade=0;trade<trades;trade++)
 {
  OrderSelect(trade,SELECT_BY_POS,MODE_TRADES);
  
  if(OrderSymbol()!=Symbol() || OrderMagicNumber() != MagicNumber )
   continue;
   
  if(OrderType()==OP_SELL)
  count++;
 }//for
 return(count);
}
//+------------------------------------------------------------------+
//| Close Long Position                                              |
//+------------------------------------------------------------------+
void CloseLongs(int MagicNumber)
{
 int i = OrdersTotal();
 
 while( CountLongs(MagicNumber) != 0 && i >= 0 )
 {  
   OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
   
   if( OrderSymbol() != Symbol() || OrderMagicNumber() != MagicNumber )
   {
     i--;
     continue;
   }
   else if(OrderType()==OP_BUY || OrderType()== OP_BUYLIMIT)
   {
     OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,ExitLongColor);
     i--;
   }
 }
}
//+------------------------------------------------------------------+
//| Close Short Position                                             |
//+------------------------------------------------------------------+
void CloseShorts(int MagicNumber)
{
 int i = OrdersTotal();
 
 while( CountShorts(MagicNumber) != 0 && i >= 0 )
 {  
   OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
   
   if( OrderSymbol() != Symbol() || OrderMagicNumber() != MagicNumber)
   {
     i--;
     continue;
   }
   else if(OrderType()== OP_SELL || OrderType()==OP_SELLLIMIT )
  {
   OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,ExitShortColor);
  }
 }
}
//+------------------------------------------------------------------+
//| Calculates Trailing Stops                                        |
//+------------------------------------------------------------------+
void TrailingAlls(int start,int stop)
{
 int profit;
 double stoptrade;
 double stopcal;
 
 if(stop==0) return;
 
 int trade;
 for(trade=OrdersTotal()-1;trade>=0;trade--)
 {
  if(!OrderSelect(trade,SELECT_BY_POS,MODE_TRADES))
   continue;

  if(OrderSymbol()!=Symbol())
   continue;

  if(OrderType()==OP_BUY)
  {
   profit=NormalizeDouble((Bid-OrderOpenPrice())/Point,0);
   if(profit<start)
    continue;
   stoptrade=OrderStopLoss();
   stopcal=Bid-(stop*Point);
   if(stoptrade==0||(stoptrade!=0&&stopcal>stoptrade))
    OrderModify(OrderTicket(),OrderOpenPrice(),stopcal,OrderTakeProfit(),0,EnterLongColor);
  }//Long
  
  if(OrderType()==OP_SELL)
  {
   profit=NormalizeDouble((OrderOpenPrice()-Ask)/Point,0);
   if(profit<start)
    continue;
   stoptrade=OrderStopLoss();
   stopcal=Ask+(stop*Point);
   if(stoptrade==0||(stoptrade!=0&&stopcal<stoptrade))
    OrderModify(OrderTicket(),OrderOpenPrice(),stopcal,OrderTakeProfit(),0,EnterShortColor);
  }//Shrt
  
 }//for
}


